@@ -49,6 +49,7 @@
 #include <ImfChannelListAttribute.h>
 #include <ImfChromaticitiesAttribute.h>
 #include <ImfCompressionAttribute.h>
+#include <ImfDeepImageStateAttribute.h>
 #include <ImfDoubleAttribute.h>
 #include <ImfEnvmapAttribute.h>
 #include <ImfFloatAttribute.h>
@@ -60,27 +61,37 @@
 #include <ImfPreviewImageAttribute.h>
 #include <ImfRationalAttribute.h>
 #include <ImfStringAttribute.h>
+#include <ImfStringVectorAttribute.h>
 #include <ImfTileDescriptionAttribute.h>
 #include <ImfTimeCodeAttribute.h>
 #include <ImfVecAttribute.h>
+#include <ImfPartType.h>
 #include "IlmThreadMutex.h"
 #include "Iex.h"
 #include <sstream>
 #include <stdlib.h>
 #include <time.h>
 
+#include "ImfNamespace.h"
 
-namespace Imf {
+OPENEXR_IMF_INTERNAL_NAMESPACE_SOURCE_ENTER
 
-using Imath::Box2i;
-using Imath::V2i;
-using Imath::V2f;
-using IlmThread::Mutex;
-using IlmThread::Lock;
+using namespace std;
+using IMATH_NAMESPACE::Box2i;
+using IMATH_NAMESPACE::V2i;
+using IMATH_NAMESPACE::V2f;
+using ILMTHREAD_NAMESPACE::Mutex;
+using ILMTHREAD_NAMESPACE::Lock;
 
 
 namespace {
 
+int maxImageWidth = 0;
+int maxImageHeight = 0;
+int maxTileWidth = 0;
+int maxTileHeight = 0;
+
+
 void
 initialize (Header &header,
 	    const Box2i &displayWindow,
@@ -101,6 +112,19 @@ initialize (Header &header,
     header.insert ("channels", ChannelListAttribute ());
 }
 
+template <size_t N>
+void checkIsNullTerminated (const char (&str)[N], const char *what)
+{
+	for (size_t i = 0; i < N; ++i) {
+		if (str[i] == '\0')
+			return;
+	}
+	std::stringstream s;
+	s << "Invalid " << what << ": it is more than " << (N - 1) 
+		<< " characters long.";
+	throw IEX_NAMESPACE::InputExc(s);
+}
+
 } // namespace
 
 
@@ -226,11 +250,32 @@ Header::operator = (const Header &other)
 }
 
 
+void
+Header::erase (const char name[])
+{
+    if (name[0] == 0)
+        THROW (IEX_NAMESPACE::ArgExc, "Image attribute name cannot be an empty string.");
+    
+    
+    AttributeMap::iterator i = _map.find (name);
+    if (i != _map.end())
+        _map.erase (i);
+
+}
+
+
+void
+Header::erase (const string &name)
+{
+    erase (name.c_str());
+}
+    
+    
 void
 Header::insert (const char name[], const Attribute &attribute)
 {
     if (name[0] == 0)
-	THROW (Iex::ArgExc, "Image attribute name cannot be an empty string.");
+	THROW (IEX_NAMESPACE::ArgExc, "Image attribute name cannot be an empty string.");
 
     AttributeMap::iterator i = _map.find (name);
 
@@ -251,7 +296,7 @@ Header::insert (const char name[], const Attribute &attribute)
     else
     {
 	if (strcmp (i->second->typeName(), attribute.typeName()))
-	    THROW (Iex::TypeExc, "Cannot assign a value of "
+	    THROW (IEX_NAMESPACE::TypeExc, "Cannot assign a value of "
 				 "type \"" << attribute.typeName() << "\" "
 				 "to image attribute \"" << name << "\" of "
 				 "type \"" << i->second->typeName() << "\".");
@@ -263,13 +308,20 @@ Header::insert (const char name[], const Attribute &attribute)
 }
 
 
+void
+Header::insert (const string &name, const Attribute &attribute)
+{
+    insert (name.c_str(), attribute);
+}
+
+
 Attribute &		
 Header::operator [] (const char name[])
 {
     AttributeMap::iterator i = _map.find (name);
 
     if (i == _map.end())
-	THROW (Iex::ArgExc, "Cannot find image attribute \"" << name << "\".");
+	THROW (IEX_NAMESPACE::ArgExc, "Cannot find image attribute \"" << name << "\".");
 
     return *i->second;
 }
@@ -281,12 +333,26 @@ Header::operator [] (const char name[]) const
     AttributeMap::const_iterator i = _map.find (name);
 
     if (i == _map.end())
-	THROW (Iex::ArgExc, "Cannot find image attribute \"" << name << "\".");
+	THROW (IEX_NAMESPACE::ArgExc, "Cannot find image attribute \"" << name << "\".");
 
     return *i->second;
 }
 
 
+Attribute &		
+Header::operator [] (const string &name)
+{
+    return this->operator[] (name.c_str());
+}
+
+
+const Attribute &	
+Header::operator [] (const string &name) const
+{
+    return this->operator[] (name.c_str());
+}
+
+
 Header::Iterator
 Header::begin ()
 {
@@ -329,7 +395,21 @@ Header::find (const char name[]) const
 }
 
 
-Imath::Box2i &	
+Header::Iterator
+Header::find (const string &name)
+{
+    return find (name.c_str());
+}
+
+
+Header::ConstIterator
+Header::find (const string &name) const
+{
+    return find (name.c_str());
+}
+
+
+IMATH_NAMESPACE::Box2i &	
 Header::displayWindow ()
 {
     return static_cast <Box2iAttribute &>
@@ -337,14 +417,15 @@ Header::displayWindow ()
 }
 
 
-const Imath::Box2i &
+const IMATH_NAMESPACE::Box2i &
 Header::displayWindow () const
 {
     return static_cast <const Box2iAttribute &>
 	((*this)["displayWindow"]).value();
 }
 
-Imath::Box2i &	
+
+IMATH_NAMESPACE::Box2i &	
 Header::dataWindow ()
 {
     return static_cast <Box2iAttribute &>
@@ -352,7 +433,7 @@ Header::dataWindow ()
 }
 
 
-const Imath::Box2i &
+const IMATH_NAMESPACE::Box2i &
 Header::dataWindow () const
 {
     return static_cast <const Box2iAttribute &>
@@ -376,7 +457,7 @@ Header::pixelAspectRatio () const
 }
 
 
-Imath::V2f &	
+IMATH_NAMESPACE::V2f &	
 Header::screenWindowCenter ()
 {
     return static_cast <V2fAttribute &>
@@ -384,7 +465,7 @@ Header::screenWindowCenter ()
 }
 
 
-const Imath::V2f &	
+const IMATH_NAMESPACE::V2f &	
 Header::screenWindowCenter () const
 {
     return static_cast <const V2fAttribute &>
@@ -456,6 +537,162 @@ Header::compression () const
 }
 
 
+void
+Header::setName(const string& name)
+{
+    insert ("name", StringAttribute (name));
+}
+
+
+bool
+Header::hasName() const
+{
+    return findTypedAttribute <StringAttribute> ("name") != 0;
+}
+
+
+string &
+Header::name()
+{
+    return typedAttribute <StringAttribute> ("name").value();
+}
+
+
+const string &
+Header::name() const
+{
+    return typedAttribute <StringAttribute> ("name").value();
+}
+
+
+void
+Header::setType(const string& type)
+{
+    if (isSupportedType(type) == false)
+    {
+        throw IEX_NAMESPACE::ArgExc (type + "is not a supported image type." +
+                           "The following are supported: " +
+                           SCANLINEIMAGE + ", " +
+                           TILEDIMAGE + ", " +
+                           DEEPSCANLINE + " or " +
+                           DEEPTILE + ".");
+    }
+
+    insert ("type", StringAttribute (type));
+
+    // (TODO) Should we do it here?
+    if (isDeepData(type) && hasVersion() == false)
+    {
+        setVersion(1);
+    }
+}
+
+
+bool
+Header::hasType() const
+{
+    return findTypedAttribute <StringAttribute> ("type") != 0;
+}
+
+
+string &
+Header::type()
+{
+    return typedAttribute <StringAttribute> ("type").value();
+}
+
+
+const string &
+Header::type() const
+{
+    return typedAttribute <StringAttribute> ("type").value();
+}
+
+
+void
+Header::setView(const string& view)
+{
+    insert ("view", StringAttribute (view));
+}
+
+
+bool
+Header::hasView() const
+{
+    return findTypedAttribute <StringAttribute> ("view") != 0;
+}
+
+
+string &
+Header::view()
+{
+    return typedAttribute <StringAttribute> ("view").value();
+}
+
+
+const string &
+Header::view() const
+{
+    return typedAttribute <StringAttribute> ("view").value();
+}
+
+
+void
+Header::setVersion(const int version)
+{
+    if (version != 1)
+    {
+        throw IEX_NAMESPACE::ArgExc ("We can only process version 1");
+    }
+
+    insert ("version", IntAttribute (version));
+}
+
+
+bool
+Header::hasVersion() const
+{
+    return findTypedAttribute <IntAttribute> ("version") != 0;
+}
+
+
+int &
+Header::version()
+{
+    return typedAttribute <IntAttribute> ("version").value();
+}
+
+
+const int &
+Header::version() const
+{
+    return typedAttribute <IntAttribute> ("version").value();
+}
+
+void 
+Header::setChunkCount(int chunks)
+{
+    insert("chunkCount",IntAttribute(chunks));
+}
+
+bool 
+Header::hasChunkCount() const
+{
+   return findTypedAttribute<IntAttribute>("chunkCount") != 0;
+}
+
+int& 
+Header::chunkCount()
+{
+    return typedAttribute <IntAttribute> ("chunkCount").value();
+}
+
+const int& 
+Header::chunkCount() const
+{
+    return typedAttribute <IntAttribute> ("chunkCount").value();
+}
+
 void
 Header::setTileDescription(const TileDescription& td)
 {
@@ -512,24 +749,65 @@ Header::hasPreviewImage () const
 
 
 void		
-Header::sanityCheck (bool isTiled) const
+Header::sanityCheck (bool isTiled, bool isMultipartFile) const
 {
     //
-    // The display window and the data window
-    // must contain at least one pixel each.
+    // The display window and the data window must each
+    // contain at least one pixel.  In addition, the
+    // coordinates of the window corners must be small
+    // enough to keep expressions like max-min+1 or
+    // max+min from overflowing.
     //
 
     const Box2i &displayWindow = this->displayWindow();
 
     if (displayWindow.min.x > displayWindow.max.x ||
-	displayWindow.min.y > displayWindow.max.y)
-	throw Iex::ArgExc ("Invalid display window in image header.");
+	displayWindow.min.y > displayWindow.max.y ||
+	displayWindow.min.x <= -(INT_MAX / 2) ||
+	displayWindow.min.y <= -(INT_MAX / 2) ||
+	displayWindow.max.x >=  (INT_MAX / 2) ||
+	displayWindow.max.y >=  (INT_MAX / 2))
+    {
+	throw IEX_NAMESPACE::ArgExc ("Invalid display window in image header.");
+    }
 
     const Box2i &dataWindow = this->dataWindow();
 
     if (dataWindow.min.x > dataWindow.max.x ||
-	dataWindow.min.y > dataWindow.max.y)
-	throw Iex::ArgExc ("Invalid data window in image header.");
+	dataWindow.min.y > dataWindow.max.y ||
+	dataWindow.min.x <= -(INT_MAX / 2) ||
+	dataWindow.min.y <= -(INT_MAX / 2) ||
+	dataWindow.max.x >=  (INT_MAX / 2) ||
+	dataWindow.max.y >=  (INT_MAX / 2))
+    {
+	throw IEX_NAMESPACE::ArgExc ("Invalid data window in image header.");
+    }
+
+    if (maxImageWidth > 0 &&
+        maxImageWidth < (dataWindow.max.x - dataWindow.min.x + 1))
+    {
+	THROW (IEX_NAMESPACE::ArgExc, "The width of the data window exceeds the "
+			    "maximum width of " << maxImageWidth << "pixels.");
+    }
+
+    if (maxImageHeight > 0 &&
+	maxImageHeight < dataWindow.max.y - dataWindow.min.y + 1)
+    {
+	THROW (IEX_NAMESPACE::ArgExc, "The width of the data window exceeds the "
+			    "maximum width of " << maxImageHeight << "pixels.");
+    }
+
+   // chunk table must be smaller than the maximum image area
+   // (only reachable for unknown types or damaged files: will have thrown earlier
+   //  for regular image types)
+   if( maxImageHeight>0 && maxImageWidth>0 && 
+       hasChunkCount() && chunkCount()>Int64(maxImageWidth)*Int64(maxImageHeight))
+   {
+       THROW (IEX_NAMESPACE::ArgExc, "chunkCount exceeds maximum area of "
+       << Int64(maxImageWidth)*Int64(maxImageHeight) << " pixels." );
+       
+   }
+
 
     //
     // The pixel aspect ratio must be greater than 0.
@@ -544,13 +822,13 @@ Header::sanityCheck (bool isTiled) const
 
     float pixelAspectRatio = this->pixelAspectRatio();
 
-    const float MIN_PIXEL_ASPECT_RATIO = 1e-6;
-    const float MAX_PIXEL_ASPECT_RATIO = 1e+6;
+    const float MIN_PIXEL_ASPECT_RATIO = 1e-6f;
+    const float MAX_PIXEL_ASPECT_RATIO = 1e+6f;
 
     if (pixelAspectRatio < MIN_PIXEL_ASPECT_RATIO ||
 	pixelAspectRatio > MAX_PIXEL_ASPECT_RATIO)
     {
-	throw Iex::ArgExc ("Invalid pixel aspect ratio in image header.");
+	throw IEX_NAMESPACE::ArgExc ("Invalid pixel aspect ratio in image header.");
     }
 
     //
@@ -564,10 +842,43 @@ Header::sanityCheck (bool isTiled) const
     float screenWindowWidth = this->screenWindowWidth();
 
     if (screenWindowWidth < 0)
-	throw Iex::ArgExc ("Invalid screen window width in image header.");
+	throw IEX_NAMESPACE::ArgExc ("Invalid screen window width in image header.");
 
     //
-    // If the file is tiled, verify that the tile description has resonable
+    // If the file has multiple parts, verify that each header has attribute
+    // name and type.
+    // (TODO) We may want to check more stuff here.
+    //
+
+    if (isMultipartFile)
+    {
+        if (!hasName())
+        {
+            throw IEX_NAMESPACE::ArgExc ("Headers in a multipart file should"
+                               " have name attribute.");
+        }
+
+        if (!hasType())
+        {
+            throw IEX_NAMESPACE::ArgExc ("Headers in a multipart file should"
+                               " have type attribute.");
+        }
+
+    }
+    
+    const std::string & part_type=hasType() ? type() : "";
+    
+    if(part_type!="" && !isSupportedType(part_type))
+    {
+        //
+        // skip remaining sanity checks with unsupported types - they may not hold
+        //
+        return;
+    }
+    
+   
+    //
+    // If the file is tiled, verify that the tile description has reasonable
     // values and check to see if the lineOrder is one of the predefined 3.
     // If the file is not tiled, then the lineOrder can only be INCREASING_Y
     // or DECREASING_Y.
@@ -579,34 +890,50 @@ Header::sanityCheck (bool isTiled) const
     {
 	if (!hasTileDescription())
 	{
-	    throw Iex::ArgExc ("Tiled image has no tile "
+	    throw IEX_NAMESPACE::ArgExc ("Tiled image has no tile "
 			       "description attribute.");
 	}
 
 	const TileDescription &tileDesc = tileDescription();
 
 	if (tileDesc.xSize <= 0 || tileDesc.ySize <= 0)
-	    throw Iex::ArgExc ("Invalid tile size in image header.");
+	    throw IEX_NAMESPACE::ArgExc ("Invalid tile size in image header.");
+
+	if (maxTileWidth > 0 &&
+	    maxTileWidth < int(tileDesc.xSize))
+	{
+	    THROW (IEX_NAMESPACE::ArgExc, "The width of the tiles exceeds the maximum "
+				"width of " << maxTileWidth << "pixels.");
+	}
+
+	if (maxTileHeight > 0 &&
+	    maxTileHeight < int(tileDesc.ySize))
+	{
+	    THROW (IEX_NAMESPACE::ArgExc, "The width of the tiles exceeds the maximum "
+				"width of " << maxTileHeight << "pixels.");
+	}
 
 	if (tileDesc.mode != ONE_LEVEL &&
 	    tileDesc.mode != MIPMAP_LEVELS &&
 	    tileDesc.mode != RIPMAP_LEVELS)
-	    throw Iex::ArgExc ("Invalid level mode in image header.");
+	    throw IEX_NAMESPACE::ArgExc ("Invalid level mode in image header.");
 
 	if (tileDesc.roundingMode != ROUND_UP &&
 	    tileDesc.roundingMode != ROUND_DOWN)
-	    throw Iex::ArgExc ("Invalid level rounding mode in image header.");
+	    throw IEX_NAMESPACE::ArgExc ("Invalid level rounding mode in image header.");
 
 	if (lineOrder != INCREASING_Y &&
 	    lineOrder != DECREASING_Y &&
 	    lineOrder != RANDOM_Y)
-	    throw Iex::ArgExc ("Invalid line order in image header.");
+	    throw IEX_NAMESPACE::ArgExc ("Invalid line order in image header.");
     }
     else
     {
-	if (lineOrder != INCREASING_Y &&
-	    lineOrder != DECREASING_Y)
-	    throw Iex::ArgExc ("Invalid line order in image header.");
+        if (lineOrder != INCREASING_Y &&
+            lineOrder != DECREASING_Y)
+            throw IEX_NAMESPACE::ArgExc ("Invalid line order in image header.");
+        
+        
     }
 
     //
@@ -614,7 +941,13 @@ Header::sanityCheck (bool isTiled) const
     //
 
     if (!isValidCompression (this->compression()))
-	throw Iex::ArgExc ("Unknown compression type in image header.");
+  	throw IEX_NAMESPACE::ArgExc ("Unknown compression type in image header.");
+    
+    if(isDeepData(part_type))
+    {
+        if (!isValidDeepCompression (this->compression()))
+            throw IEX_NAMESPACE::ArgExc ("Compression type in header not valid for deep data");
+    }
 
     //
     // Check the channel list:
@@ -637,24 +970,24 @@ Header::sanityCheck (bool isTiled) const
 	     i != channels.end();
 	     ++i)
 	{
-	    if (i.channel().type != UINT &&
-		i.channel().type != HALF &&
-		i.channel().type != FLOAT)
+	    if (i.channel().type != OPENEXR_IMF_INTERNAL_NAMESPACE::UINT &&
+		    i.channel().type != OPENEXR_IMF_INTERNAL_NAMESPACE::HALF &&
+		    i.channel().type != OPENEXR_IMF_INTERNAL_NAMESPACE::FLOAT)
 	    {
-		THROW (Iex::ArgExc, "Pixel type of \"" << i.name() << "\" "
+		THROW (IEX_NAMESPACE::ArgExc, "Pixel type of \"" << i.name() << "\" "
 			            "image channel is invalid.");
 	    }
 
 	    if (i.channel().xSampling != 1)
 	    {
-		THROW (Iex::ArgExc, "The x subsampling factor for the "
+		THROW (IEX_NAMESPACE::ArgExc, "The x subsampling factor for the "
 				    "\"" << i.name() << "\" channel "
 				    "is not 1.");
 	    }	
 
 	    if (i.channel().ySampling != 1)
 	    {
-		THROW (Iex::ArgExc, "The y subsampling factor for the "
+		THROW (IEX_NAMESPACE::ArgExc, "The y subsampling factor for the "
 				    "\"" << i.name() << "\" channel "
 				    "is not 1.");
 	    }	
@@ -666,31 +999,31 @@ Header::sanityCheck (bool isTiled) const
 	     i != channels.end();
 	     ++i)
 	{
-	    if (i.channel().type != UINT &&
-		i.channel().type != HALF &&
-		i.channel().type != FLOAT)
+	    if (i.channel().type != OPENEXR_IMF_INTERNAL_NAMESPACE::UINT &&
+		    i.channel().type != OPENEXR_IMF_INTERNAL_NAMESPACE::HALF &&
+		    i.channel().type != OPENEXR_IMF_INTERNAL_NAMESPACE::FLOAT)
 	    {
-		THROW (Iex::ArgExc, "Pixel type of \"" << i.name() << "\" "
+		THROW (IEX_NAMESPACE::ArgExc, "Pixel type of \"" << i.name() << "\" "
 			            "image channel is invalid.");
 	    }
 
 	    if (i.channel().xSampling < 1)
 	    {
-		THROW (Iex::ArgExc, "The x subsampling factor for the "
+		THROW (IEX_NAMESPACE::ArgExc, "The x subsampling factor for the "
 				    "\"" << i.name() << "\" channel "
 				    "is invalid.");
 	    }
 
 	    if (i.channel().ySampling < 1)
 	    {
-		THROW (Iex::ArgExc, "The y subsampling factor for the "
+		THROW (IEX_NAMESPACE::ArgExc, "The y subsampling factor for the "
 				    "\"" << i.name() << "\" channel "
 				    "is invalid.");
 	    }
 
 	    if (dataWindow.min.x % i.channel().xSampling)
 	    {
-		THROW (Iex::ArgExc, "The minimum x coordinate of the "
+		THROW (IEX_NAMESPACE::ArgExc, "The minimum x coordinate of the "
 				    "image's data window is not a multiple "
 				    "of the x subsampling factor of "
 				    "the \"" << i.name() << "\" channel.");
@@ -698,7 +1031,7 @@ Header::sanityCheck (bool isTiled) const
 
 	    if (dataWindow.min.y % i.channel().ySampling)
 	    {
-		THROW (Iex::ArgExc, "The minimum y coordinate of the "
+		THROW (IEX_NAMESPACE::ArgExc, "The minimum y coordinate of the "
 				    "image's data window is not a multiple "
 				    "of the y subsampling factor of "
 				    "the \"" << i.name() << "\" channel.");
@@ -707,7 +1040,7 @@ Header::sanityCheck (bool isTiled) const
 	    if ((dataWindow.max.x - dataWindow.min.x + 1) %
 		    i.channel().xSampling)
 	    {
-		THROW (Iex::ArgExc, "Number of pixels per row in the "
+		THROW (IEX_NAMESPACE::ArgExc, "Number of pixels per row in the "
 				    "image's data window is not a multiple "
 				    "of the x subsampling factor of "
 				    "the \"" << i.name() << "\" channel.");
@@ -716,7 +1049,7 @@ Header::sanityCheck (bool isTiled) const
 	    if ((dataWindow.max.y - dataWindow.min.y + 1) %
 		    i.channel().ySampling)
 	    {
-		THROW (Iex::ArgExc, "Number of pixels per column in the "
+		THROW (IEX_NAMESPACE::ArgExc, "Number of pixels per column in the "
 				    "image's data window is not a multiple "
 				    "of the y subsampling factor of "
 				    "the \"" << i.name() << "\" channel.");
@@ -726,18 +1059,38 @@ Header::sanityCheck (bool isTiled) const
 }
 
 
+void		
+Header::setMaxImageSize (int maxWidth, int maxHeight)
+{
+    maxImageWidth = maxWidth;
+    maxImageHeight = maxHeight;
+}
+
+
+void		
+Header::setMaxTileSize (int maxWidth, int maxHeight)
+{
+    maxTileWidth = maxWidth;
+    maxTileHeight = maxHeight;
+}
+
+
+bool
+Header::readsNothing()
+{
+    return _readsNothing;
+}
+
+
 Int64
-Header::writeTo (OStream &os, bool isTiled) const
+Header::writeTo (OPENEXR_IMF_INTERNAL_NAMESPACE::OStream &os, bool isTiled) const
 {
     //
     // Write a "magic number" to identify the file as an image file.
     // Write the current file format version number.
     //
 
-    Xdr::write <StreamIO> (os, MAGIC);
-
-    int version = isTiled ? makeTiled (EXR_VERSION) : EXR_VERSION;
-    Xdr::write <StreamIO> (os, version);
+    int version = EXR_VERSION;
 
     //
     // Write all attributes.  If we have a preview image attribute,
@@ -755,8 +1108,8 @@ Header::writeTo (OStream &os, bool isTiled) const
 	// Write the attribute's name and type.
 	//
 
-	Xdr::write <StreamIO> (os, i.name());
-	Xdr::write <StreamIO> (os, i.attribute().typeName());
+	OPENEXR_IMF_INTERNAL_NAMESPACE::Xdr::write <OPENEXR_IMF_INTERNAL_NAMESPACE::StreamIO> (os, i.name());
+	OPENEXR_IMF_INTERNAL_NAMESPACE::Xdr::write <OPENEXR_IMF_INTERNAL_NAMESPACE::StreamIO> (os, i.attribute().typeName());
 
 	//
 	// Write the size of the attribute value,
@@ -767,60 +1120,33 @@ Header::writeTo (OStream &os, bool isTiled) const
 	i.attribute().writeValueTo (oss, version);
 
 	std::string s = oss.str();
-	Xdr::write <StreamIO> (os, (int) s.length());
+	OPENEXR_IMF_INTERNAL_NAMESPACE::Xdr::write <OPENEXR_IMF_INTERNAL_NAMESPACE::StreamIO> (os, (int) s.length());
 
 	if (&i.attribute() == preview)
 	    previewPosition = os.tellp();
 
-	os.write (s.data(), s.length());
+	os.write (s.data(), int(s.length()));
     }
 
     //
     // Write zero-length attribute name to mark the end of the header.
     //
 
-    Xdr::write <StreamIO> (os, "");
+    OPENEXR_IMF_INTERNAL_NAMESPACE::Xdr::write <OPENEXR_IMF_INTERNAL_NAMESPACE::StreamIO> (os, "");
 
     return previewPosition;
 }
 
 
 void
-Header::readFrom (IStream &is, int &version)
+Header::readFrom (OPENEXR_IMF_INTERNAL_NAMESPACE::IStream &is, int &version)
 {
-    //
-    // Read the magic number and the file format version number.
-    // Then check if we can read the rest of this file.
-    //
-
-    int magic;
-
-    Xdr::read <StreamIO> (is, magic);
-    Xdr::read <StreamIO> (is, version);
-
-    if (magic != MAGIC)
-    {
-	throw Iex::InputExc ("File is not an image file.");
-    }
-
-    if (getVersion (version) != EXR_VERSION)
-    {
-	THROW (Iex::InputExc, "Cannot read "
-			      "version " << getVersion (version) << " "
-			      "image files.  Current file format version "
-			      "is " << EXR_VERSION << ".");
-    }
-    
-    if (!supportsFlags (getFlags (version)))
-    {
-	THROW (Iex::InputExc, "The file format version number's flag field "
-			      "contains unrecognized flags.");
-    }
-
     //
     // Read all attributes.
     //
 
+    int attrCount = 0;
+
     while (true)
     {
 	//
@@ -828,21 +1154,30 @@ Header::readFrom (IStream &is, int &version)
 	// A zero-length attribute name indicates the end of the header.
 	//
 
-	char name[100];
-	Xdr::read <StreamIO> (is, sizeof (name), name);
+	char name[Name::SIZE];
+	OPENEXR_IMF_INTERNAL_NAMESPACE::Xdr::read <OPENEXR_IMF_INTERNAL_NAMESPACE::StreamIO> (is, Name::MAX_LENGTH, name);
 
 	if (name[0] == 0)
+	{
+	    if (attrCount == 0) _readsNothing = true;
+	    else                _readsNothing = false;
 	    break;
+	}
+
+	attrCount++;
+
+	checkIsNullTerminated (name, "attribute name");
 
 	//
 	// Read the attribute type and the size of the attribute value.
 	//
 
-	char typeName[100];
+	char typeName[Name::SIZE];
 	int size;
 
-	Xdr::read <StreamIO> (is, sizeof (typeName), typeName);
-	Xdr::read <StreamIO> (is, size);
+	OPENEXR_IMF_INTERNAL_NAMESPACE::Xdr::read <OPENEXR_IMF_INTERNAL_NAMESPACE::StreamIO> (is, Name::MAX_LENGTH, typeName);
+	checkIsNullTerminated (typeName, "attribute type name");
+	OPENEXR_IMF_INTERNAL_NAMESPACE::Xdr::read <OPENEXR_IMF_INTERNAL_NAMESPACE::StreamIO> (is, size);
 
 	AttributeMap::iterator i = _map.find (name);
 
@@ -855,7 +1190,7 @@ Header::readFrom (IStream &is, int &version)
 	    //
 
 	    if (strncmp (i->second->typeName(), typeName, sizeof (typeName)))
-		THROW (Iex::InputExc, "Unexpected type for image attribute "
+		THROW (IEX_NAMESPACE::InputExc, "Unexpected type for image attribute "
 				      "\"" << name << "\".");
 
 	    i->second->readValueFrom (is, size, version);
@@ -912,21 +1247,27 @@ staticInitialize ()
 	ChannelListAttribute::registerAttributeType();
 	CompressionAttribute::registerAttributeType();
 	ChromaticitiesAttribute::registerAttributeType();
+	DeepImageStateAttribute::registerAttributeType();
 	DoubleAttribute::registerAttributeType();
 	EnvmapAttribute::registerAttributeType();
 	FloatAttribute::registerAttributeType();
 	IntAttribute::registerAttributeType();
 	KeyCodeAttribute::registerAttributeType();
 	LineOrderAttribute::registerAttributeType();
+	M33dAttribute::registerAttributeType();
 	M33fAttribute::registerAttributeType();
+	M44dAttribute::registerAttributeType();
 	M44fAttribute::registerAttributeType();
 	PreviewImageAttribute::registerAttributeType();
 	RationalAttribute::registerAttributeType();
 	StringAttribute::registerAttributeType();
+        StringVectorAttribute::registerAttributeType();
 	TileDescriptionAttribute::registerAttributeType();
 	TimeCodeAttribute::registerAttributeType();
+	V2dAttribute::registerAttributeType();
 	V2fAttribute::registerAttributeType();
 	V2iAttribute::registerAttributeType();
+	V3dAttribute::registerAttributeType();
 	V3fAttribute::registerAttributeType();
 	V3iAttribute::registerAttributeType();
 
@@ -935,4 +1276,4 @@ staticInitialize ()
 }
 
 
-} // namespace Imf
+OPENEXR_IMF_INTERNAL_NAMESPACE_SOURCE_EXIT
