@@ -1,3 +1,7 @@
+//
+///\todo: version needs fixing!
+//
+
 ///////////////////////////////////////////////////////////////////////////
 //
 // Copyright (c) 2004, Industrial Light & Magic, a division of Lucas
@@ -48,34 +52,40 @@
 #include "ImathBox.h"
 #include "ImathFun.h"
 #include <ImfArray.h>
-#include <ImfXdr.h>
+#include "ImfXdr.h"
 #include <ImfPreviewImageAttribute.h>
+#include <ImfPartType.h>
 #include "IlmThreadPool.h"
+#include "ImfOutputStreamMutex.h"
 #include "IlmThreadSemaphore.h"
 #include "IlmThreadMutex.h"
 #include "Iex.h"
+#include "ImfInputPart.h"
+#include "ImfNamespace.h"
+#include "ImfOutputPartData.h"
+
 #include <string>
 #include <vector>
 #include <fstream>
 #include <assert.h>
 
+OPENEXR_IMF_INTERNAL_NAMESPACE_SOURCE_ENTER
 
-namespace Imf {
-
-using Imath::Box2i;
-using Imath::divp;
-using Imath::modp;
+using IMATH_NAMESPACE::Box2i;
+using IMATH_NAMESPACE::divp;
+using IMATH_NAMESPACE::modp;
 using std::string;
 using std::vector;
 using std::ofstream;
 using std::min;
 using std::max;
-using IlmThread::Mutex;
-using IlmThread::Lock;
-using IlmThread::Semaphore;
-using IlmThread::Task;
-using IlmThread::TaskGroup;
-using IlmThread::ThreadPool;
+using ILMTHREAD_NAMESPACE::Mutex;
+using ILMTHREAD_NAMESPACE::Lock;
+using ILMTHREAD_NAMESPACE::Semaphore;
+using ILMTHREAD_NAMESPACE::Task;
+using ILMTHREAD_NAMESPACE::TaskGroup;
+using ILMTHREAD_NAMESPACE::ThreadPool;
+
 
 namespace {
 
@@ -97,6 +107,7 @@ struct OutSliceInfo
 	          int xSampling = 1,
 	          int ySampling = 1,
 	          bool zero = false);
+                  
 };
 
 
@@ -165,11 +176,11 @@ LineBuffer::~LineBuffer ()
 
 } // namespace
 
-
-struct OutputFile::Data: public Mutex
+struct OutputFile::Data
 {
     Header		 header;		// the image header
-    int			 version;		// file format version
+    bool                 multiPart;		// is the file multipart?
+    int			 version;               // version attribute \todo NOT BEING WRITTEN PROPERLY
     Int64		 previewPosition;       // file position for preview
     FrameBuffer		 frameBuffer;           // framebuffer to write into
     int			 currentScanLine;       // next scanline to be written
@@ -187,18 +198,18 @@ struct OutputFile::Data: public Mutex
                                                 // its linebuffer
     Compressor::Format	 format;                // compressor's data format
     vector<OutSliceInfo> slices;		// info about channels in file
-    OStream *		 os;			// file stream to write to
-    bool		 deleteStream;
     Int64		 lineOffsetsPosition;   // file position for line
                                                 // offset table
-    Int64		 currentPosition;       // current file position
 
     vector<LineBuffer*>  lineBuffers;           // each holds one line buffer
     int			 linesInBuffer;         // number of scanlines each
                                                 // buffer holds
     size_t		 lineBufferSize;        // size of the line buffer
 
-     Data (bool deleteStream, int numThreads);
+    int                  partNumber;            // the output part number
+    OutputStreamMutex *  _streamData;         
+    bool                 _deleteStream;
+     Data (int numThreads);
     ~Data ();
 
 
@@ -208,10 +219,11 @@ struct OutputFile::Data: public Mutex
 };
 
 
-OutputFile::Data::Data (bool deleteStream, int numThreads):
-    os (0),
-    deleteStream (deleteStream),
-    lineOffsetsPosition (0)
+OutputFile::Data::Data (int numThreads):
+    lineOffsetsPosition (0),
+    partNumber (-1),
+    _streamData(0),
+    _deleteStream(false)
 {
     //
     // We need at least one lineBuffer, but if threading is used,
@@ -224,9 +236,6 @@ OutputFile::Data::Data (bool deleteStream, int numThreads):
 
 OutputFile::Data::~Data ()
 {
-    if (deleteStream)
-	delete os;
-
     for (size_t i = 0; i < lineBuffers.size(); i++)
         delete lineBuffers[i];
 }
@@ -238,30 +247,29 @@ OutputFile::Data::getLineBuffer (int number)
     return lineBuffers[number % lineBuffers.size()];
 }
 
-
 namespace {
 
-
 Int64
-writeLineOffsets (OStream &os, const vector<Int64> &lineOffsets)
+writeLineOffsets (OPENEXR_IMF_INTERNAL_NAMESPACE::OStream &os, const vector<Int64> &lineOffsets)
 {
     Int64 pos = os.tellp();
 
     if (pos == -1)
-	Iex::throwErrnoExc ("Cannot determine current file position (%T).");
-
+	IEX_NAMESPACE::throwErrnoExc ("Cannot determine current file position (%T).");
+    
     for (unsigned int i = 0; i < lineOffsets.size(); i++)
-	Xdr::write <StreamIO> (os, lineOffsets[i]);
+	Xdr::write<StreamIO> (os, lineOffsets[i]);
 
     return pos;
 }
 
 
 void
-writePixelData (OutputFile::Data *ofd,
+writePixelData (OutputStreamMutex *filedata,
+                OutputFile::Data *partdata,
                 int lineBufferMinY,
-		const char pixelData[],
-		int pixelDataSize)
+                const char pixelData[],
+                int pixelDataSize)
 {
     //
     // Store a block of pixel data in the output file, and try
@@ -269,39 +277,53 @@ writePixelData (OutputFile::Data *ofd,
     // without calling tellp() (tellp() can be fairly expensive).
     //
 
-    Int64 currentPosition = ofd->currentPosition;
-    ofd->currentPosition = 0;
+    Int64 currentPosition = filedata->currentPosition;
+    filedata->currentPosition = 0;
 
     if (currentPosition == 0)
-	currentPosition = ofd->os->tellp();
+        currentPosition = filedata->os->tellp();
 
-    ofd->lineOffsets[(ofd->currentScanLine - ofd->minY) / ofd->linesInBuffer] =
-	currentPosition;
+    partdata->lineOffsets[(partdata->currentScanLine - partdata->minY) / partdata->linesInBuffer] =
+        currentPosition;
 
     #ifdef DEBUG
 
-	assert (ofd->os->tellp() == currentPosition);
+        assert (filedata->os->tellp() == currentPosition);
 
     #endif
+    
+	
+	
+    if (partdata->multiPart)
+    {
+        OPENEXR_IMF_INTERNAL_NAMESPACE::Xdr::write <OPENEXR_IMF_INTERNAL_NAMESPACE::StreamIO> (*filedata->os, partdata->partNumber);
+    }
+    
+    OPENEXR_IMF_INTERNAL_NAMESPACE::Xdr::write <OPENEXR_IMF_INTERNAL_NAMESPACE::StreamIO> (*filedata->os, lineBufferMinY);
+    OPENEXR_IMF_INTERNAL_NAMESPACE::Xdr::write <OPENEXR_IMF_INTERNAL_NAMESPACE::StreamIO> (*filedata->os, pixelDataSize);
+    filedata->os->write (pixelData, pixelDataSize);
 
-    Xdr::write <StreamIO> (*ofd->os, lineBufferMinY);
-    Xdr::write <StreamIO> (*ofd->os, pixelDataSize);
-    ofd->os->write (pixelData, pixelDataSize);
+    filedata->currentPosition = currentPosition +
+                           Xdr::size<int>() +
+                           Xdr::size<int>() +
+                           pixelDataSize;
 
-    ofd->currentPosition = currentPosition +
-			   Xdr::size<int>() +
-			   Xdr::size<int>() +
-			   pixelDataSize;
+    if (partdata->multiPart)
+    {
+        filedata->currentPosition += Xdr::size<int>();
+    }
 }
 
 
 inline void
-writePixelData (OutputFile::Data *ofd, const LineBuffer *lineBuffer)
+writePixelData (OutputStreamMutex* filedata,
+                OutputFile::Data *partdata,
+                const LineBuffer *lineBuffer)
 {
-    writePixelData (ofd,
-		    lineBuffer->minY,
+    writePixelData (filedata, partdata,
+                    lineBuffer->minY,
                     lineBuffer->dataPtr,
-		    lineBuffer->dataSize);
+                    lineBuffer->dataSize);
 }
 
 
@@ -326,35 +348,18 @@ convertToXdr (OutputFile::Data *ofd,
     // pixel data in place, without an intermediate temporary buffer.
     //
    
-    int startY, endY;		// The first and last scanlines in
-    				// the file that are in the lineBuffer.
-    int step;
-    
-    if (ofd->lineOrder == INCREASING_Y)
-    {
-	startY = max (lineBufferMinY, ofd->minY);
-	endY = min (lineBufferMaxY, ofd->maxY) + 1;
-        step = 1;
-    }
-    else
-    {
-	startY = min (lineBufferMaxY, ofd->maxY);
-	endY = max (lineBufferMinY, ofd->minY) - 1;
-        step = -1;
-    }
-
     //
     // Iterate over all scanlines in the lineBuffer to convert.
     //
 
-    for (int y = startY; y != endY; y += step)
+    char *writePtr = &lineBuffer[0];
+    for (int y = lineBufferMinY; y <= lineBufferMaxY; y++)
     {
 	//
         // Set these to point to the start of line y.
         // We will write to writePtr from readPtr.
 	//
-	
-        char *writePtr = lineBuffer + ofd->offsetInLineBuffer[y - ofd->minY];
+
         const char *readPtr = writePtr;
         
 	//
@@ -507,7 +512,7 @@ LineBufferTask::execute ()
             //
             // Iterate over all image channels.
             //
-        
+
             for (unsigned int i = 0; i < _ofd->slices.size(); ++i)
             {
                 //
@@ -650,54 +655,133 @@ OutputFile::OutputFile
      const Header &header,
      int numThreads)
 :
-    _data (new Data (true, numThreads))
+    _data (new Data (numThreads))
+    
 {
+    _data->_streamData=new OutputStreamMutex ();
+    _data->_deleteStream=true;
     try
     {
 	header.sanityCheck();
-	_data->os = new StdOFStream (fileName);
+	_data->_streamData->os = new StdOFStream (fileName);
+        _data->multiPart=false; // only one header, not multipart
 	initialize (header);
+	_data->_streamData->currentPosition = _data->_streamData->os->tellp();
+        
+	// Write header and empty offset table to the file.
+	writeMagicNumberAndVersionField(*_data->_streamData->os, _data->header);
+	_data->previewPosition =
+	        _data->header.writeTo (*_data->_streamData->os);
+        _data->lineOffsetsPosition =
+                writeLineOffsets (*_data->_streamData->os,_data->lineOffsets);
     }
-    catch (Iex::BaseExc &e)
+    catch (IEX_NAMESPACE::BaseExc &e)
     {
-	delete _data;
+        if (_data && _data->_streamData) delete _data->_streamData;
+	if (_data)       delete _data;
 
 	REPLACE_EXC (e, "Cannot open image file "
 			"\"" << fileName << "\". " << e);
 	throw;
     }
+    catch (...)
+    {
+        if (_data && _data->_streamData) delete _data->_streamData;
+        if (_data)       delete _data;
+
+        throw;
+    }
 }
 
 
 OutputFile::OutputFile
-    (OStream &os,
+    (OPENEXR_IMF_INTERNAL_NAMESPACE::OStream &os,
      const Header &header,
      int numThreads)
 :
-    _data (new Data (false, numThreads))
+    _data (new Data (numThreads))
 {
+    
+    _data->_streamData=new OutputStreamMutex ();
+    _data->_deleteStream=false;
     try
     {
 	header.sanityCheck();
-	_data->os = &os;
+	_data->_streamData->os = &os;
+        _data->multiPart=false;
 	initialize (header);
+	_data->_streamData->currentPosition = _data->_streamData->os->tellp();
+
+	// Write header and empty offset table to the file.
+	writeMagicNumberAndVersionField(*_data->_streamData->os, _data->header);
+	_data->previewPosition =
+	        _data->header.writeTo (*_data->_streamData->os);
+        _data->lineOffsetsPosition =
+                writeLineOffsets (*_data->_streamData->os, _data->lineOffsets);
     }
-    catch (Iex::BaseExc &e)
+    catch (IEX_NAMESPACE::BaseExc &e)
     {
-	delete _data;
+        if (_data && _data->_streamData) delete _data->_streamData;
+	if (_data)       delete _data;
 
 	REPLACE_EXC (e, "Cannot open image file "
 			"\"" << os.fileName() << "\". " << e);
 	throw;
     }
+    catch (...)
+    {
+        if (_data && _data->_streamData) delete _data->_streamData;
+        if (_data)       delete _data;
+
+        throw;
+    }
 }
 
+OutputFile::OutputFile(const OutputPartData* part) : _data(NULL)
+{
+    try
+    {
+        if (part->header.type() != SCANLINEIMAGE)
+            throw IEX_NAMESPACE::ArgExc("Can't build a OutputFile from a type-mismatched part.");
+
+        _data = new Data (part->numThreads);
+        _data->_streamData = part->mutex;
+        _data->_deleteStream=false;
+        _data->multiPart=part->multipart;
+
+        initialize (part->header);
+        _data->partNumber = part->partNumber;
+        _data->lineOffsetsPosition = part->chunkOffsetTablePosition;
+        _data->previewPosition = part->previewPosition;
+    }
+    catch (IEX_NAMESPACE::BaseExc &e)
+    {
+        if (_data) delete _data;
+
+        REPLACE_EXC (e, "Cannot initialize output part "
+                        "\"" << part->partNumber << "\". " << e);
+        throw;
+    }
+    catch (...)
+    {
+        if (_data) delete _data;
+
+        throw;
+    }
+}
 
 void
 OutputFile::initialize (const Header &header)
 {
     _data->header = header;
 
+    // "fix" the type if it happens to be set incorrectly
+    // (attribute is optional, but ensure it is correct if it exists)
+    if(_data->header.hasType())
+    {
+        _data->header.setType(SCANLINEIMAGE);
+    }
+    
     const Box2i &dataWindow = header.dataWindow();
 
     _data->currentScanLine = (header.lineOrder() == INCREASING_Y)?
@@ -733,18 +817,11 @@ OutputFile::initialize (const Header &header)
 			  _data->linesInBuffer) / _data->linesInBuffer;
 
     _data->lineOffsets.resize (lineOffsetSize);
-
+    
+    
     offsetInLineBufferTable (_data->bytesPerLine,
 			     _data->linesInBuffer,
 			     _data->offsetInLineBuffer);
-
-    _data->previewPosition =
-	_data->header.writeTo (*_data->os);
-
-    _data->lineOffsetsPosition =
-	writeLineOffsets (*_data->os, _data->lineOffsets);
-
-    _data->currentPosition = _data->os->tellp();
 }
 
 
@@ -753,12 +830,20 @@ OutputFile::~OutputFile ()
     if (_data)
     {
         {
+            Lock lock(*_data->_streamData);
+            Int64 originalPosition = _data->_streamData->os->tellp();
+
             if (_data->lineOffsetsPosition > 0)
             {
                 try
                 {
-                    _data->os->seekp (_data->lineOffsetsPosition);
-                    writeLineOffsets (*_data->os, _data->lineOffsets);
+                    _data->_streamData->os->seekp (_data->lineOffsetsPosition);
+                    writeLineOffsets (*_data->_streamData->os, _data->lineOffsets);
+
+                    //
+                    // Restore the original position.
+                    //
+                    _data->_streamData->os->seekp (originalPosition);
                 }
                 catch (...)
                 {
@@ -772,15 +857,22 @@ OutputFile::~OutputFile ()
             }
         }
 
+        if (_data->_deleteStream && _data->_streamData)
+            delete _data->_streamData->os;
+
+        if (_data->partNumber == -1 && _data->_streamData)
+            delete _data->_streamData;
+
 	delete _data;
     }
+
 }
 
 
 const char *
 OutputFile::fileName () const
 {
-    return _data->os->fileName();
+    return _data->_streamData->os->fileName();
 }
 
 
@@ -794,7 +886,7 @@ OutputFile::header () const
 void	
 OutputFile::setFrameBuffer (const FrameBuffer &frameBuffer)
 {
-    Lock lock (*_data);
+    Lock lock (*_data->_streamData);
     
     //
     // Check if the new frame buffer descriptor
@@ -814,7 +906,7 @@ OutputFile::setFrameBuffer (const FrameBuffer &frameBuffer)
 
 	if (i.channel().type != j.slice().type)
 	{
-	    THROW (Iex::ArgExc, "Pixel type of \"" << i.name() << "\" channel "
+	    THROW (IEX_NAMESPACE::ArgExc, "Pixel type of \"" << i.name() << "\" channel "
 			        "of output file \"" << fileName() << "\" is "
 			        "not compatible with the frame buffer's "
 			        "pixel type.");
@@ -823,7 +915,7 @@ OutputFile::setFrameBuffer (const FrameBuffer &frameBuffer)
 	if (i.channel().xSampling != j.slice().xSampling ||
 	    i.channel().ySampling != j.slice().ySampling)
 	{
-	    THROW (Iex::ArgExc, "X and/or y subsampling factors "
+	    THROW (IEX_NAMESPACE::ArgExc, "X and/or y subsampling factors "
 				"of \"" << i.name() << "\" channel "
 				"of output file \"" << fileName() << "\" are "
 				"not compatible with the frame buffer's "
@@ -886,7 +978,7 @@ OutputFile::setFrameBuffer (const FrameBuffer &frameBuffer)
 const FrameBuffer &
 OutputFile::frameBuffer () const
 {
-    Lock lock (*_data);
+    Lock lock (*_data->_streamData);
     return _data->frameBuffer;
 }
 
@@ -896,10 +988,10 @@ OutputFile::writePixels (int numScanLines)
 {
     try
     {
-        Lock lock (*_data);
+        Lock lock (*_data->_streamData);
 
 	if (_data->slices.size() == 0)
-	    throw Iex::ArgExc ("No frame buffer specified "
+	    throw IEX_NAMESPACE::ArgExc ("No frame buffer specified "
 			       "as pixel data source.");
 
         //
@@ -985,7 +1077,7 @@ OutputFile::writePixels (int numScanLines)
             {
                 if (_data->missingScanLines <= 0)
                 {
-                    throw Iex::ArgExc ("Tried to write more scan lines "
+                    throw IEX_NAMESPACE::ArgExc ("Tried to write more scan lines "
                                        "than specified by the data window.");
                 }
     
@@ -1021,7 +1113,7 @@ OutputFile::writePixels (int numScanLines)
                 // Write the line buffer
 		//
 
-                writePixelData (_data, writeBuffer);
+                writePixelData (_data->_streamData, _data, writeBuffer);
                 nextWriteBuffer += step;
 
                 _data->currentScanLine = _data->currentScanLine +
@@ -1094,7 +1186,7 @@ OutputFile::writePixels (int numScanLines)
 
 	const string *exception = 0;
 
-        for (int i = 0; i < _data->lineBuffers.size(); ++i)
+        for (size_t i = 0; i < _data->lineBuffers.size(); ++i)
 	{
             LineBuffer *lineBuffer = _data->lineBuffers[i];
 
@@ -1105,9 +1197,9 @@ OutputFile::writePixels (int numScanLines)
 	}
 
 	if (exception)
-	    throw Iex::IoExc (*exception);
+	    throw IEX_NAMESPACE::IoExc (*exception);
     }
-    catch (Iex::BaseExc &e)
+    catch (IEX_NAMESPACE::BaseExc &e)
     {
 	REPLACE_EXC (e, "Failed to write pixel data to image "
 		        "file \"" << fileName() << "\". " << e);
@@ -1119,7 +1211,7 @@ OutputFile::writePixels (int numScanLines)
 int	
 OutputFile::currentScanLine () const
 {
-    Lock lock (*_data);
+    Lock lock (*_data->_streamData);
     return _data->currentScanLine;
 }
 
@@ -1127,7 +1219,7 @@ OutputFile::currentScanLine () const
 void	
 OutputFile::copyPixels (InputFile &in)
 {
-    Lock lock (*_data);
+    Lock lock (*_data->_streamData);
 
     //
     // Check if this file's and and the InputFile's
@@ -1138,7 +1230,7 @@ OutputFile::copyPixels (InputFile &in)
     const Header &inHdr = in.header();
 
     if (inHdr.find("tiles") != inHdr.end())
-	THROW (Iex::ArgExc, "Cannot copy pixels from image "
+	THROW (IEX_NAMESPACE::ArgExc, "Cannot copy pixels from image "
 			    "file \"" << in.fileName() << "\" to image "
 			    "file \"" << fileName() << "\". "
                             "The input file is tiled, but the output file is "
@@ -1146,25 +1238,25 @@ OutputFile::copyPixels (InputFile &in)
                             "instead.");
 
     if (!(hdr.dataWindow() == inHdr.dataWindow()))
-	THROW (Iex::ArgExc, "Cannot copy pixels from image "
+	THROW (IEX_NAMESPACE::ArgExc, "Cannot copy pixels from image "
 			    "file \"" << in.fileName() << "\" to image "
 			    "file \"" << fileName() << "\". "
                             "The files have different data windows.");
 
     if (!(hdr.lineOrder() == inHdr.lineOrder()))
-	THROW (Iex::ArgExc, "Quick pixel copy from image "
+	THROW (IEX_NAMESPACE::ArgExc, "Quick pixel copy from image "
 			    "file \"" << in.fileName() << "\" to image "
 			    "file \"" << fileName() << "\" failed. "
 			    "The files have different line orders.");
 
     if (!(hdr.compression() == inHdr.compression()))
-	THROW (Iex::ArgExc, "Quick pixel copy from image "
+	THROW (IEX_NAMESPACE::ArgExc, "Quick pixel copy from image "
 			    "file \"" << in.fileName() << "\" to image "
 			    "file \"" << fileName() << "\" failed. "
 			    "The files use different compression methods.");
 
     if (!(hdr.channels() == inHdr.channels()))
-	THROW (Iex::ArgExc, "Quick pixel copy from image "
+	THROW (IEX_NAMESPACE::ArgExc, "Quick pixel copy from image "
 			    "file \"" << in.fileName() << "\" to image "
 			    "file \"" << fileName() << "\" failed.  "
 			    "The files have different channel lists.");
@@ -1176,7 +1268,7 @@ OutputFile::copyPixels (InputFile &in)
     const Box2i &dataWindow = hdr.dataWindow();
 
     if (_data->missingScanLines != dataWindow.max.y - dataWindow.min.y + 1)
-	THROW (Iex::LogicExc, "Quick pixel copy from image "
+	THROW (IEX_NAMESPACE::LogicExc, "Quick pixel copy from image "
 			      "file \"" << in.fileName() << "\" to image "
 			      "file \"" << fileName() << "\" failed. "
 			      "\"" << fileName() << "\" already contains "
@@ -1193,9 +1285,9 @@ OutputFile::copyPixels (InputFile &in)
 
 	in.rawPixelData (_data->currentScanLine, pixelData, pixelDataSize);
 
-	writePixelData (_data, lineBufferMinY (_data->currentScanLine,
-				               _data->minY,
-				               _data->linesInBuffer),
+        writePixelData (_data->_streamData, _data, lineBufferMinY (_data->currentScanLine,
+                                               _data->minY,
+                                               _data->linesInBuffer),
                         pixelData, pixelDataSize);
 
 	_data->currentScanLine += (_data->lineOrder == INCREASING_Y)?
@@ -1206,13 +1298,21 @@ OutputFile::copyPixels (InputFile &in)
 }
 
 
+void
+OutputFile::copyPixels( InputPart & in)
+{
+    copyPixels(*in.file);
+}
+
+
+
 void
 OutputFile::updatePreviewImage (const PreviewRgba newPixels[])
 {
-    Lock lock (*_data);
+    Lock lock (*_data->_streamData);
 
     if (_data->previewPosition <= 0)
-	THROW (Iex::LogicExc, "Cannot update preview image pixels. "
+	THROW (IEX_NAMESPACE::LogicExc, "Cannot update preview image pixels. "
 			      "File \"" << fileName() << "\" does not "
 			      "contain a preview image.");
 
@@ -1236,15 +1336,15 @@ OutputFile::updatePreviewImage (const PreviewRgba newPixels[])
     // preview image, and jump back to the saved file position.
     //
 
-    Int64 savedPosition = _data->os->tellp();
+    Int64 savedPosition = _data->_streamData->os->tellp();
 
     try
     {
-	_data->os->seekp (_data->previewPosition);
-	pia.writeValueTo (*_data->os, _data->version);
-	_data->os->seekp (savedPosition);
+        _data->_streamData->os->seekp (_data->previewPosition);
+	pia.writeValueTo (*_data->_streamData->os, _data->version);
+	_data->_streamData->os->seekp (savedPosition);
     }
-    catch (Iex::BaseExc &e)
+    catch (IEX_NAMESPACE::BaseExc &e)
     {
 	REPLACE_EXC (e, "Cannot update preview image pixels for "
 			"file \"" << fileName() << "\". " << e);
@@ -1256,22 +1356,22 @@ OutputFile::updatePreviewImage (const PreviewRgba newPixels[])
 void	
 OutputFile::breakScanLine  (int y, int offset, int length, char c)
 {
-    Lock lock (*_data);
+    Lock lock (*_data->_streamData);
 
     Int64 position = 
 	_data->lineOffsets[(y - _data->minY) / _data->linesInBuffer];
 
     if (!position)
-	THROW (Iex::ArgExc, "Cannot overwrite scan line " << y << ". "
+	THROW (IEX_NAMESPACE::ArgExc, "Cannot overwrite scan line " << y << ". "
 			    "The scan line has not yet been stored in "
 			    "file \"" << fileName() << "\".");
 
-    _data->currentPosition = 0;
-    _data->os->seekp (position + offset);
+    _data->_streamData->currentPosition = 0;
+    _data->_streamData->os->seekp (position + offset);
 
     for (int i = 0; i < length; ++i)
-	_data->os->write (&c, 1);
+        _data->_streamData->os->write (&c, 1);
 }
 
 
-} // namespace Imf
+OPENEXR_IMF_INTERNAL_NAMESPACE_SOURCE_EXIT
