@@ -53,7 +53,7 @@
 #include <iostream>
 
 #include "namespaceAlias.h"
-using namespace CustomImf;
+using namespace IMF;
 using namespace std;
 using namespace IMATH_NAMESPACE;
 
