@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004, Industrial Light & Magic, a division of Lucas
+// Copyright (c) 2004-2012, Industrial Light & Magic, a division of Lucas
 // Digital Ltd. LLC
 // 
 // All rights reserved.
@@ -34,6 +34,7 @@
 
 
 
+#include <tmpDir.h>
 
 #include <ImfOutputFile.h>
 #include <ImfInputFile.h>
@@ -48,6 +49,7 @@
 #include <ImfChromaticitiesAttribute.h>
 #include <ImfFloatAttribute.h>
 #include <ImfEnvmapAttribute.h>
+#include <ImfDeepImageStateAttribute.h>
 #include <ImfDoubleAttribute.h>
 #include <ImfIntAttribute.h>
 #include <ImfLineOrderAttribute.h>
@@ -60,9 +62,10 @@
 #include <stdio.h>
 #include <assert.h>
 
-#include "tmpDir.h"
 
-using namespace OPENEXR_IMF_NAMESPACE;
+namespace IMF = OPENEXR_IMF_NAMESPACE;
+using namespace IMF;
+
 using namespace std;
 using namespace IMATH_NAMESPACE;
 
@@ -109,9 +112,6 @@ writeReadAttr (const Array2D<float> &pf1,
     a15.push_back ("");
     a15.push_back ("straw into");
     a15.push_back ("gold");
-
-    
-    
     M33d   a16 (12, 13, 14, 15, 16, 17, 18, 19, 20);
     M44d   a17 (2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17);
     V2d    a18 (27.51, 28.51);
@@ -119,6 +119,8 @@ writeReadAttr (const Array2D<float> &pf1,
     
     StringVector a20;
 
+    DeepImageState a21 (DIS_TIDY);
+
     //
     // Write an image file with extra attributes in the header
     //
@@ -145,10 +147,11 @@ writeReadAttr (const Array2D<float> &pf1,
 	hdr.insert ("a17", M44dAttribute   (a17));
 	hdr.insert ("a18", V2dAttribute    (a18));
 	hdr.insert ("a19", V3dAttribute    (a19));
-          hdr.insert ("a20", StringVectorAttribute  (a20));
+	hdr.insert ("a20", StringVectorAttribute  (a20));
+	hdr.insert ("a21", DeepImageStateAttribute  (a21));
 
 	hdr.channels().insert ("F",			// name
-			       Channel (FLOAT,		// type
+			       Channel (IMF::FLOAT,	// type
 					1,		// xSampling
 					1)		// ySampling
 			      );
@@ -156,7 +159,7 @@ writeReadAttr (const Array2D<float> &pf1,
 	FrameBuffer fb; 
 
 	fb.insert ("F",					// name
-		   Slice (FLOAT,			// type
+		   Slice (IMF::FLOAT,			// type
 			  (char *) &pf1[0][0],		// base
 			  sizeof (pf1[0][0]), 		// xStride
 			  sizeof (pf1[0][0]) * width,	// yStride
@@ -232,9 +235,10 @@ writeReadAttr (const Array2D<float> &pf1,
 	assert (hdr.typedAttribute <M44dAttribute>   ("a17").value()  == a17);
 	assert (hdr.typedAttribute <V2dAttribute>    ("a18").value()  == a18);
 	assert (hdr.typedAttribute <V3dAttribute>    ("a19").value()  == a19);
-          assert (hdr.typedAttribute <StringVectorAttribute>    ("a20").value()  == a20);
-          
-          
+	assert (hdr.typedAttribute <StringVectorAttribute>
+                                                  ("a20").value() == a20);
+	assert (hdr.typedAttribute <DeepImageStateAttribute>
+				                                  ("a21").value() == a21);
     }
 
     remove (fileName);
@@ -252,11 +256,11 @@ channelList ()
 
 	ChannelList channels;
 
-	channels.insert ("b0", Channel (HALF, 1, 1));
-	channels.insert ("b1", Channel (HALF, 1, 1));
-	channels.insert ("b2", Channel (HALF, 1, 1));
-	channels.insert ("d3", Channel (HALF, 1, 1));
-	channels.insert ("e4", Channel (HALF, 1, 1));
+	channels.insert ("b0", Channel (IMF::HALF, 1, 1));
+	channels.insert ("b1", Channel (IMF::HALF, 1, 1));
+	channels.insert ("b2", Channel (IMF::HALF, 1, 1));
+	channels.insert ("d3", Channel (IMF::HALF, 1, 1));
+	channels.insert ("e4", Channel (IMF::HALF, 1, 1));
 
 	ChannelList::Iterator first;
 	ChannelList::Iterator last;
@@ -309,24 +313,24 @@ channelList ()
 	
 	ChannelList channels;
 
-	channels.insert ("a",   Channel (HALF, 1, 1));
-	channels.insert (".a",  Channel (HALF, 1, 1));
-	channels.insert ("a.",  Channel (HALF, 1, 1));
+	channels.insert ("a",   Channel (IMF::HALF, 1, 1));
+	channels.insert (".a",  Channel (IMF::HALF, 1, 1));
+	channels.insert ("a.",  Channel (IMF::HALF, 1, 1));
 
-	channels.insert ("layer1.R", Channel (HALF, 1, 1));
-	channels.insert ("layer1.G", Channel (HALF, 1, 1));
-	channels.insert ("layer1.B", Channel (HALF, 1, 1));
+	channels.insert ("layer1.R", Channel (IMF::HALF, 1, 1));
+	channels.insert ("layer1.G", Channel (IMF::HALF, 1, 1));
+	channels.insert ("layer1.B", Channel (IMF::HALF, 1, 1));
 
-	channels.insert ("layer1.sublayer1.AA", Channel (HALF, 1, 1));
-	channels.insert ("layer1.sublayer1.R", Channel (HALF, 1, 1));
-	channels.insert ("layer1.sublayer1.G", Channel (HALF, 1, 1));
-	channels.insert ("layer1.sublayer1.B", Channel (HALF, 1, 1));
+	channels.insert ("layer1.sublayer1.AA", Channel (IMF::HALF, 1, 1));
+	channels.insert ("layer1.sublayer1.R", Channel  (IMF::HALF, 1, 1));
+	channels.insert ("layer1.sublayer1.G", Channel  (IMF::HALF, 1, 1));
+	channels.insert ("layer1.sublayer1.B", Channel  (IMF::HALF, 1, 1));
 
-	channels.insert ("layer1.sublayer2.R", Channel (HALF, 1, 1));
+	channels.insert ("layer1.sublayer2.R", Channel (IMF::HALF, 1, 1));
 
-	channels.insert ("layer2.R", Channel (HALF, 1, 1));
-	channels.insert ("layer2.G", Channel (HALF, 1, 1));
-	channels.insert ("layer2.B", Channel (HALF, 1, 1));
+	channels.insert ("layer2.R", Channel (IMF::HALF, 1, 1));
+	channels.insert ("layer2.G", Channel (IMF::HALF, 1, 1));
+	channels.insert ("layer2.B", Channel (IMF::HALF, 1, 1));
 
 	set <string> layerNames;
 	channels.layers (layerNames);
@@ -374,7 +378,7 @@ longNames (const Array2D<float> &pf1,
     FrameBuffer fb; 
 
     fb.insert ("F",					// name
-               Slice (FLOAT,				// type
+               Slice (IMF::FLOAT,			// type
                       (char *) &pf1[0][0],		// base
                       sizeof (pf1[0][0]), 		// xStride
                       sizeof (pf1[0][0]) * width,	// yStride
@@ -388,7 +392,7 @@ longNames (const Array2D<float> &pf1,
 	Header hdr (width, height);
 
 	hdr.channels().insert ("F",			// name
-			       Channel (FLOAT,		// type
+			       Channel (IMF::FLOAT,	// type
 					1,		// xSampling
 					1)		// ySampling
 			      );
@@ -418,7 +422,7 @@ longNames (const Array2D<float> &pf1,
 	hdr.insert (longName, StringAttribute ("y"));
 
 	hdr.channels().insert ("F",			// name
-			       Channel (FLOAT,		// type
+			       Channel (IMF::FLOAT,	// type
 					1,		// xSampling
 					1)		// ySampling
 			      );
@@ -447,7 +451,7 @@ longNames (const Array2D<float> &pf1,
 	Header hdr (width, height);
 
 	hdr.channels().insert (longName,		// name
-			       Channel (FLOAT,		// type
+			       Channel (IMF::FLOAT,	// type
 					1,		// xSampling
 					1)		// ySampling
 			      );
@@ -485,7 +489,7 @@ print_type(const OPENEXR_IMF_NAMESPACE::TypedAttribute<T> & object)
 }
 
 void
-testAttributes ()
+testAttributes (const std::string &tempDir)
 {
     try
     {
@@ -497,14 +501,14 @@ testAttributes ()
 	Array2D<float> pf (H, W);
 	fillPixels (pf, W, H);
 
-	const char *filename = IMF_TMP_DIR "imf_test_attr.exr";
+	std::string filename = tempDir + "imf_test_attr.exr";
 
-	writeReadAttr (pf, filename, W, H);
+	writeReadAttr (pf, filename.c_str(), W, H);
 	channelList();
-        longNames(pf, filename, W, H);
+        longNames(pf, filename.c_str(), W, H);
+
+    print_type(OPENEXR_IMF_NAMESPACE::TypedAttribute<int>());
 
-        print_type(OPENEXR_IMF_NAMESPACE::TypedAttribute<int>());
-        
 	cout << "ok\n" << endl;
     }
     catch (const std::exception &e)
