@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004, Industrial Light & Magic, a division of Lucas
+// Copyright (c) 2004-2012, Industrial Light & Magic, a division of Lucas
 // Digital Ltd. LLC
 // 
 // All rights reserved.
@@ -46,9 +46,8 @@
 #include <stdio.h>
 #include <assert.h>
 
-#include "tmpDir.h"
-
-using namespace OPENEXR_IMF_NAMESPACE;
+namespace IMF = OPENEXR_IMF_NAMESPACE;
+using namespace IMF;
 using namespace std;
 using namespace IMATH_NAMESPACE;
 
@@ -180,19 +179,19 @@ writeRead (const Array2D<unsigned int> &pi1,
     hdr.compression() = comp;
 
     hdr.channels().insert ("I",			// name
-			   Channel (UINT,	// type
+			   Channel (IMF::UINT,	// type
 				    xs,		// xSampling
 				    ys)		// ySampling
 			  );
 
     hdr.channels().insert ("H",			// name
-			   Channel (HALF,	// type
+			   Channel (IMF::HALF,	// type
 				    xs,		// xSampling
 				    ys)		// ySampling
 			  );
 
     hdr.channels().insert ("F",			// name
-			   Channel (FLOAT,	// type
+			   Channel (IMF::FLOAT,	// type
 				    xs,		// xSampling
 				    ys)		// ySampling
 			  );
@@ -201,7 +200,7 @@ writeRead (const Array2D<unsigned int> &pi1,
 	FrameBuffer fb; 
 
 	fb.insert ("I",						// name
-		   Slice (UINT,					// type
+		   Slice (IMF::UINT,				// type
 			  (char *) &pi1[-yOffset / ys][-xOffset / xs], // base
 			  sizeof (pi1[0][0]), 			// xStride
 			  sizeof (pi1[0][0]) * (width / xs),	// yStride
@@ -210,7 +209,7 @@ writeRead (const Array2D<unsigned int> &pi1,
 		  );
 	
 	fb.insert ("H",						// name
-		   Slice (HALF,					// type
+		   Slice (IMF::HALF,				// type
 			  (char *) &ph1[-yOffset / ys][-xOffset / xs], // base
 			  sizeof (ph1[0][0]), 			// xStride
 			  sizeof (ph1[0][0]) * (width / xs),	// yStride
@@ -219,7 +218,7 @@ writeRead (const Array2D<unsigned int> &pi1,
 		  );
 	
 	fb.insert ("F",						// name
-		   Slice (FLOAT,				// type
+		   Slice (IMF::FLOAT,				// type
 			  (char *) &pf1[-yOffset / ys][-xOffset / xs], // base
 			  sizeof (pf1[0][0]), 			// xStride
 			  sizeof (pf1[0][0]) * (width / xs),	// yStride
@@ -257,7 +256,7 @@ writeRead (const Array2D<unsigned int> &pi1,
 	    int ys = in.header().channels()["I"].ySampling;
 
 	    fb.insert ("I",					// name
-		       Slice (UINT,				// type
+		       Slice (IMF::UINT,			// type
 			      (char *) &pi2[-dy / ys][-dx / xs], // base
 			      sizeof (pi2[0][0]), 		// xStride
 			      sizeof (pi2[0][0]) * (w / xs),	// yStride
@@ -271,7 +270,7 @@ writeRead (const Array2D<unsigned int> &pi1,
 	    int ys = in.header().channels()["H"].ySampling;
 
 	    fb.insert ("H",					// name
-		       Slice (HALF,				// type
+		       Slice (IMF::HALF,			// type
 			      (char *) &ph2[-dy / ys][-dx / xs], // base
 			      sizeof (ph2[0][0]), 		// xStride
 			      sizeof (ph2[0][0]) * (w / xs),	// yStride
@@ -285,7 +284,7 @@ writeRead (const Array2D<unsigned int> &pi1,
 	    int ys = in.header().channels()["F"].ySampling;
 
 	    fb.insert ("F",					// name
-		       Slice (FLOAT,				// type
+		       Slice (IMF::FLOAT,			// type
 			      (char *) &pf2[-dy / ys][-dx / xs], // base
 			      sizeof (pf2[0][0]), 		// xStride
 			      sizeof (pf2[0][0]) * (w / xs),	// yStride
@@ -353,7 +352,8 @@ writeRead (const Array2D<unsigned int> &pi1,
 
 
 void
-writeRead (const Array2D<unsigned int> &pi,
+writeRead (const std::string &tempDir,
+           const Array2D<unsigned int> &pi,
 	   const Array2D<half> &ph,
 	   const Array2D<float> &pf,
 	   int w,
@@ -361,7 +361,7 @@ writeRead (const Array2D<unsigned int> &pi,
 	   int dx,
 	   int dy)
 {
-    const char *filename = IMF_TMP_DIR "imf_test_comp.exr";
+    std::string filename = tempDir + "imf_test_comp.exr";
 
     for (int xs = 1; xs <= 2; ++xs)
     {
@@ -370,7 +370,7 @@ writeRead (const Array2D<unsigned int> &pi,
 	    for (int comp = 0; comp < NUM_COMPRESSION_METHODS; ++comp)
 	    {
 		writeRead (pi, ph, pf,
-			   filename,
+			   filename.c_str(),
 			   w  * xs, h  * ys,
 			   dx * xs, dy * ys,
 			   Compression (comp),
@@ -384,7 +384,7 @@ writeRead (const Array2D<unsigned int> &pi,
 
 
 void
-testCompression ()
+testCompression (const std::string &tempDir)
 {
     try
     {
@@ -410,16 +410,16 @@ testCompression ()
 	assert (NUM_PIXELTYPES == 3);
 
 	fillPixels1 (pi, ph, pf, W, H);
-	writeRead (pi, ph, pf, W, H, DX, DY);
+	writeRead (tempDir, pi, ph, pf, W, H, DX, DY);
 
 	fillPixels2 (pi, ph, pf, W, H);
-	writeRead (pi, ph, pf, W, H, DX, DY);
+	writeRead (tempDir, pi, ph, pf, W, H, DX, DY);
 
 	fillPixels3 (pi, ph, pf, W, H);
-	writeRead (pi, ph, pf, W, H, DX, DY);
+	writeRead (tempDir, pi, ph, pf, W, H, DX, DY);
 
 	fillPixels4 (pi, ph, pf, W, H);
-	writeRead (pi, ph, pf, W, H, DX, DY);
+	writeRead (tempDir, pi, ph, pf, W, H, DX, DY);
 
 	cout << "ok\n" << endl;
     }
