@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004, Industrial Light & Magic, a division of Lucas
+// Copyright (c) 2004-2012, Industrial Light & Magic, a division of Lucas
 // Digital Ltd. LLC
 // 
 // All rights reserved.
@@ -33,8 +33,7 @@
 ///////////////////////////////////////////////////////////////////////////
 
 
-#include <tmpDir.h>
-#include <compareB44.h>
+#include "compareB44.h"
 
 #include <ImfRgbaFile.h>
 #include <ImfArray.h>
@@ -45,9 +44,11 @@
 #include <ImfThreading.h>
 #include <IlmThread.h>
 
-using namespace Imf;
-using namespace Imath;
+
+using namespace OPENEXR_IMF_NAMESPACE;
 using namespace std;
+using namespace IMATH_NAMESPACE;
+
 
 namespace {
 
@@ -140,7 +141,8 @@ writeReadRGBA (const char fileName[],
 	assert (in.compression() == header.compression());
 	assert (in.channels() == channels);
 
-	if (in.compression() == B44_COMPRESSION)
+	if (in.compression() == B44_COMPRESSION ||
+	    in.compression() == B44A_COMPRESSION)
 	{
 	    compareB44 (w, h, p1, p2, channels);
 	}
@@ -182,13 +184,13 @@ writeReadRGBA (const char fileName[],
 
 
 void
-testRgbaThreading ()
+testRgbaThreading (const std::string &tempDir)
 {
     try
     {
         cout << "Testing setGlobalThreadCount()" << endl;
 
-        if (!IlmThread::supportsThreads ())
+        if (!ILMTHREAD_NAMESPACE::supportsThreads ())
         {
             cout << "   Threading not supported!" << endl << endl;
             return;
@@ -225,25 +227,25 @@ testRgbaThreading ()
             {
                 for (int lorder = 0; lorder < RANDOM_Y; ++lorder)
                 {
-                    writeReadRGBA (IMF_TMP_DIR "imf_test_rgba.exr",
+                    writeReadRGBA ((tempDir + "imf_test_rgba.exr").c_str(),
                                    W, H, p1,
                                    WRITE_RGBA,
                                    LineOrder (lorder),
                                    Compression (comp));
     
-                    writeReadRGBA (IMF_TMP_DIR "imf_test_rgba.exr",
+                    writeReadRGBA ((tempDir + "imf_test_rgba.exr").c_str(),
                                    W, H, p1,
                                    WRITE_RGB,
                                    LineOrder (lorder),
                                    Compression (comp));
     
-                    writeReadRGBA ("imf_test_rgba.exr",
+                    writeReadRGBA ((tempDir + "imf_test_rgba.exr").c_str(),
                                    W, H, p1,
                                    WRITE_A,
                                    LineOrder (lorder),
                                    Compression (comp));
     
-                    writeReadRGBA ("imf_test_rgba.exr",
+                    writeReadRGBA ((tempDir + "imf_test_rgba.exr").c_str(),
                                    W, H, p1,
                                    RgbaChannels (WRITE_R | WRITE_B),
                                    LineOrder (lorder),
