@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004, Industrial Light & Magic, a division of Lucas
+// Copyright (c) 2004-2012, Industrial Light & Magic, a division of Lucas
 // Digital Ltd. LLC
 // 
 // All rights reserved.
@@ -33,8 +33,7 @@
 ///////////////////////////////////////////////////////////////////////////
 
 
-#include <tmpDir.h>
-#include <compareB44.h>
+#include "compareB44.h"
 
 #include <ImfOutputFile.h>
 #include <ImfInputFile.h>
@@ -45,14 +44,17 @@
 #include <half.h>
 #include <ImathRandom.h>
 #include <ImfTileDescriptionAttribute.h>
-#include <compareFloat.h>
+#include <ImfNamespace.h>
+
+#include "compareFloat.h"
 
 #include <stdio.h>
 #include <assert.h>
 
+namespace IMF = OPENEXR_IMF_NAMESPACE;
+using namespace IMF;
 using namespace std;
-using namespace Imath;
-using namespace Imf;
+using namespace IMATH_NAMESPACE;
 
 
 namespace {
@@ -188,9 +190,9 @@ writeRead (const Array2D<unsigned int> &pi1,
     hdr.compression() = comp;
     hdr.lineOrder() = lorder;
 
-    hdr.channels().insert ("I", Channel (UINT));
-    hdr.channels().insert ("H", Channel (HALF));
-    hdr.channels().insert ("F", Channel (FLOAT));
+    hdr.channels().insert ("I", Channel (IMF::UINT));
+    hdr.channels().insert ("H", Channel (IMF::HALF));
+    hdr.channels().insert ("F", Channel (IMF::FLOAT));
     
     hdr.setTileDescription(TileDescription(xSize, ySize, ONE_LEVEL));
     
@@ -198,21 +200,21 @@ writeRead (const Array2D<unsigned int> &pi1,
         FrameBuffer fb; 
 
         fb.insert ("I",                                       // name
-                   Slice (UINT,                               // type
+                   Slice (IMF::UINT,                               // type
                           (char *) &pi1[-yOffset][-xOffset],  // base
                           sizeof (pi1[0][0]),                 // xStride
                           sizeof (pi1[0][0]) * width)         // yStride
                   );
                   
         fb.insert ("H",                                       // name
-                   Slice (HALF,                               // type
+                   Slice (IMF::HALF,                               // type
                           (char *) &ph1[-yOffset][-xOffset],  // base
                           sizeof (ph1[0][0]),                 // xStride
                           sizeof (ph1[0][0]) * width)         // yStride
                   );
                   
         fb.insert ("F",                                       // name
-                   Slice (FLOAT,                              // type
+                   Slice (IMF::FLOAT,                              // type
                           (char *) &pf1[-yOffset][-xOffset],  // base
                           sizeof (pf1[0][0]),                 // xStride
                           sizeof (pf1[0][0]) * width)         // yStride
@@ -244,21 +246,21 @@ writeRead (const Array2D<unsigned int> &pi1,
         FrameBuffer fb;
 
         fb.insert ("I",                             // name
-                   Slice (UINT,                     // type
+                   Slice (IMF::UINT,                     // type
                           (char *) &pi2[-dwy][-dwx],// base
                           sizeof (pi2[0][0]),       // xStride
                           sizeof (pi2[0][0]) * w)   // yStride
                   );
 
         fb.insert ("H",                             // name
-                   Slice (HALF,                     // type
+                   Slice (IMF::HALF,                     // type
                           (char *) &ph2[-dwy][-dwx],// base
                           sizeof (ph2[0][0]),       // xStride
                           sizeof (ph2[0][0]) * w)   // yStride
                   );
 
         fb.insert ("F",                             // name
-                   Slice (FLOAT,                    // type
+                   Slice (IMF::FLOAT,                    // type
                           (char *) &pf2[-dwy][-dwx],// base
                           sizeof (pf2[0][0]),       // xStride
                           sizeof (pf2[0][0]) * w)   // yStride
@@ -294,7 +296,7 @@ writeRead (const Array2D<unsigned int> &pi1,
 
         assert (ii == in.header().channels().end());
 
-	if (comp == B44_COMPRESSION)
+	if (comp == B44_COMPRESSION || comp == B44A_COMPRESSION)
 	{
 	    for (int y = 0; y < h; y += ySize)
 	    {
@@ -326,7 +328,7 @@ writeRead (const Array2D<unsigned int> &pi1,
 	    {
 		assert (pi1[y][x] == pi2[y][x]);
 
-		if (comp != B44_COMPRESSION)
+		if (comp != B44_COMPRESSION && comp != B44A_COMPRESSION)
 		    assert (ph1[y][x] == ph2[y][x]);
 
 		assert (equivalent (pf1[y][x], pf2[y][x], comp));
@@ -346,7 +348,7 @@ writeRead (const Array2D<unsigned int> &pi1,
         FrameBuffer fb;
 
         fb.insert ("I",                                 // name
-                   Slice (UINT,                         // type
+                   Slice (IMF::UINT,                         // type
                           (char *) &pi2[0][0],          // base
                           sizeof (pi2[0][0]),           // xStride
                           sizeof (pi2[0][0]) * xSize,   // yStride
@@ -358,7 +360,7 @@ writeRead (const Array2D<unsigned int> &pi1,
                   );
 
         fb.insert ("H",                                 // name
-                   Slice (HALF,                         // type
+                   Slice (IMF::HALF,                         // type
                           (char *) &ph2[0][0],          // base
                           sizeof (ph2[0][0]),           // xStride
                           sizeof (ph2[0][0]) * xSize,   // yStride
@@ -370,7 +372,7 @@ writeRead (const Array2D<unsigned int> &pi1,
                   );
 
         fb.insert ("F",                                 // name
-                   Slice (FLOAT,                        // type
+                   Slice (IMF::FLOAT,                        // type
                           (char *) &pf2[0][0],          // base
                           sizeof (pf2[0][0]),           // xStride
                           sizeof (pf2[0][0]) * xSize,   // yStride
@@ -413,7 +415,7 @@ writeRead (const Array2D<unsigned int> &pi1,
             {
                 in.readTile (tileX, tileY);
 
-                Imath::Box2i win = in.dataWindowForTile (tileX, tileY);
+                IMATH_NAMESPACE::Box2i win = in.dataWindowForTile (tileX, tileY);
                 int oX = win.min.x - xOffset;
                 int oY = win.min.y - yOffset;
 
@@ -425,8 +427,10 @@ writeRead (const Array2D<unsigned int> &pi1,
                     {
                         assert (pi1[oY + y][oX + x] == pi2[y][x]);
 
-			if (comp != B44_COMPRESSION)
+			if (comp != B44_COMPRESSION && comp != B44A_COMPRESSION)
+			{
 			    assert (ph1[oY + y][oX + x] == ph2[y][x]);
+			}
 
                         assert (equivalent (pf1[oY + y][oX + x],
                                             pf2[y][x], comp));
@@ -442,7 +446,8 @@ writeRead (const Array2D<unsigned int> &pi1,
 
 
 void
-writeRead (const Array2D<unsigned int> &pi,
+writeRead (const std::string &tempDir,
+           const Array2D<unsigned int> &pi,
            const Array2D<half> &ph,
            const Array2D<float> &pf,
            int w,
@@ -452,12 +457,12 @@ writeRead (const Array2D<unsigned int> &pi,
            int dx,
            int dy)
 {
-    const char *filename = IMF_TMP_DIR "imf_test_comp.exr";
+    std::string filename = tempDir + "imf_test_comp.exr";
 
     for (int comp = 0; comp < NUM_COMPRESSION_METHODS; ++comp)
     {
         writeRead (pi, ph, pf,
-                   filename,
+                   filename.c_str(),
                    LineOrder (0),
                    w, h,
                    xs, ys,
@@ -470,7 +475,7 @@ writeRead (const Array2D<unsigned int> &pi,
 
 
 void
-testTiledCompression ()
+testTiledCompression (const std::string &tempDir)
 {
     try
     {
@@ -503,16 +508,16 @@ testTiledCompression ()
 		    "yOffset = " << DY[i] << endl;
 
             fillPixels1 (pi, ph, pf, W, H);
-            writeRead (pi, ph, pf, W, H, XS, YS, DX[i], DY[i]);
+            writeRead (tempDir, pi, ph, pf, W, H, XS, YS, DX[i], DY[i]);
 
             fillPixels2 (pi, ph, pf, W, H);
-            writeRead (pi, ph, pf, W, H, XS, YS, DX[i], DY[i]);
+            writeRead (tempDir, pi, ph, pf, W, H, XS, YS, DX[i], DY[i]);
 
             fillPixels3 (pi, ph, pf, W, H);
-            writeRead (pi, ph, pf, W, H, XS, YS, DX[i], DY[i]);
+            writeRead (tempDir, pi, ph, pf, W, H, XS, YS, DX[i], DY[i]);
 
             fillPixels4 (pi, ph, pf, W, H);
-            writeRead (pi, ph, pf, W, H, XS, YS, DX[i], DY[i]);
+            writeRead (tempDir, pi, ph, pf, W, H, XS, YS, DX[i], DY[i]);
         }
 
         cout << "ok\n" << endl;
