@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004, Industrial Light & Magic, a division of Lucas
+// Copyright (c) 2004-2012, Industrial Light & Magic, a division of Lucas
 // Digital Ltd. LLC
 // 
 // All rights reserved.
@@ -33,8 +33,6 @@
 ///////////////////////////////////////////////////////////////////////////
 
 
-#include <tmpDir.h>
-
 #include <ImfTiledOutputFile.h>
 #include <ImfInputFile.h>
 #include <ImathRandom.h>
@@ -49,9 +47,11 @@
 #include <stdio.h>
 #include <assert.h>
 
+
+using namespace OPENEXR_IMF_NAMESPACE;
 using namespace std;
-using namespace Imath;
-using namespace Imf;
+using namespace IMATH_NAMESPACE;
+
 
 namespace {
 
@@ -580,13 +580,13 @@ writeCopyReadRIP (const char fileName[],
 
 
 void
-writeCopyRead (int w, int h, int xs, int ys)
+writeCopyRead (const std::string &tempDir, int w, int h, int xs, int ys)
 {
-    const char *filename = IMF_TMP_DIR "imf_test_copy.exr";
+    std::string filename = tempDir + "imf_test_copy.exr";
 
     for (int comp = 0; comp < NUM_COMPRESSION_METHODS; ++comp)
     {
-	if (comp == B44_COMPRESSION)
+	if (comp == B44_COMPRESSION || comp == B44A_COMPRESSION)
 	    continue;
 
         for (int lorder = 0; lorder < RANDOM_Y; ++lorder)
@@ -597,21 +597,21 @@ writeCopyRead (int w, int h, int xs, int ys)
 		{
 		    for (int ts = 0; ts <= 1; ++ts)
 		    {
-			writeCopyReadONE (filename, w, h,
+			writeCopyReadONE (filename.c_str(), w, h,
 					  (LineOrder)lorder,
 					  (LevelRoundingMode) rmode,
 					  xs, ys,
 					  Compression (comp),
 					  (bool)tb, (bool)ts);
 
-			writeCopyReadMIP (filename, w, h,
+			writeCopyReadMIP (filename.c_str(), w, h,
 					  (LineOrder)lorder,
 					  (LevelRoundingMode) rmode,
 					  xs, ys,
 					  Compression (comp),
 					  (bool)tb, (bool)ts);
 
-			writeCopyReadRIP (filename, w, h,
+			writeCopyReadRIP (filename.c_str(), w, h,
 					  (LineOrder)lorder,
 					  (LevelRoundingMode) rmode,
 					  xs, ys,
@@ -628,7 +628,7 @@ writeCopyRead (int w, int h, int xs, int ys)
 
 
 void
-testTiledLineOrder ()
+testTiledLineOrder (const std::string &tempDir)
 {
     try
     {
@@ -640,17 +640,17 @@ testTiledLineOrder ()
         const int XS = 55;
         const int YS = 55;
 
-	int maxThreads = IlmThread::supportsThreads()? 3: 0;
+	int maxThreads = ILMTHREAD_NAMESPACE::supportsThreads()? 3: 0;
 
 	for (int n = 0; n <= maxThreads; ++n)
 	{
-	    if (IlmThread::supportsThreads())
+	    if (ILMTHREAD_NAMESPACE::supportsThreads())
 	    {
 		setGlobalThreadCount (n);
 		cout << "\nnumber of threads: " << globalThreadCount() << endl;
 	    }
 
-	    writeCopyRead (W, H, XS, YS);
+	    writeCopyRead (tempDir, W, H, XS, YS);
 	}
 
         cout << "ok\n" << endl;
