@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////
 //
-// Copyright (c) 2004, Industrial Light & Magic, a division of Lucas
+// Copyright (c) 2004-2012, Industrial Light & Magic, a division of Lucas
 // Digital Ltd. LLC
 // 
 // All rights reserved.
@@ -33,10 +33,12 @@
 ///////////////////////////////////////////////////////////////////////////
 
 
-#include <tmpDir.h>
-#include <compareB44.h>
+#include "compareB44.h"
 
 #include <ImfTiledRgbaFile.h>
+#include <ImfTiledOutputFile.h>
+#include <ImfChannelList.h>
+#include <ImfFrameBuffer.h>
 #include <ImfArray.h>
 #include <ImfThreading.h>
 #include <IlmThread.h>
@@ -47,9 +49,11 @@
 #include <vector>
 #include <math.h>
 
-using namespace Imf;
-using namespace Imath;
+
+using namespace OPENEXR_IMF_NAMESPACE;
 using namespace std;
+using namespace IMATH_NAMESPACE;
+
 
 namespace {
 
@@ -418,11 +422,11 @@ writeReadRGBARIP (const char fileName[],
 
 
 void
-writeRead (int W, int H, Compression comp, int xSize, int ySize)
+writeRead (const std::string &tempDir, int W, int H, Compression comp, int xSize, int ySize)
 {
-    const char *filename = IMF_TMP_DIR "imf_test_tiled_rgba.exr";
+    std::string filename = tempDir + "imf_test_tiled_rgba.exr";
 
-    writeReadRGBAONE (filename, W, H, WRITE_RGBA, comp, xSize, ySize);
+    writeReadRGBAONE (filename.c_str(), W, H, WRITE_RGBA, comp, xSize, ySize);
 
     if (comp != B44_COMPRESSION && comp != B44A_COMPRESSION)
     {
@@ -432,18 +436,18 @@ writeRead (int W, int H, Compression comp, int xSize, int ySize)
 	// to verify that B44 compression works with tiled files.
 	//
 
-	writeReadRGBAMIP (filename, W, H, WRITE_RGBA, comp, xSize, ySize);
-	writeReadRGBARIP (filename, W, H, WRITE_RGBA, comp, xSize, ySize);
+	writeReadRGBAMIP (filename.c_str(), W, H, WRITE_RGBA, comp, xSize, ySize);
+	writeReadRGBARIP (filename.c_str(), W, H, WRITE_RGBA, comp, xSize, ySize);
     }
 }
 
 
 void
-writeReadIncomplete ()
+writeReadIncomplete (const std::string &tempDir)
 {
     cout << "\nfile with missing and broken tiles" << endl;
 
-    const char *fileName = IMF_TMP_DIR "imf_test_tiled_incomplete.exr";
+    std::string fileName = tempDir + "imf_test_tiled_incomplete.exr";
 
     //
     // Write a file where every other tile is missing or broken.
@@ -465,12 +469,12 @@ writeReadIncomplete ()
     {
         cout << "writing" << endl;
  
-        remove (fileName);
+        remove (fileName.c_str());
 
 	Header header (width, height);
 	header.lineOrder() = RANDOM_Y;
 
-        TiledRgbaOutputFile out (fileName, header, WRITE_RGBA,
+        TiledRgbaOutputFile out (fileName.c_str(), header, WRITE_RGBA,
                                  tileXSize, tileYSize, ONE_LEVEL);
         
         out.setFrameBuffer (&p1[0][0], 1, width);
@@ -497,7 +501,7 @@ writeReadIncomplete ()
 
         cout << "reading one tile at a time," << flush;
 
-        TiledRgbaInputFile in (fileName);
+        TiledRgbaInputFile in (fileName.c_str());
         const Box2i &dw = in.dataWindow();
 
         assert (dw.max.x - dw.min.x + 1 == width);
@@ -518,11 +522,11 @@ writeReadIncomplete ()
 		{
 		    in.readTile (tileX, tileY);
 		}
-		catch (const Iex::InputExc &)
+		catch (const IEX_NAMESPACE::InputExc &)
 		{
 		    tilePresent = false;	// tile is missing
 		}
-		catch (const Iex::IoExc &)
+		catch (const IEX_NAMESPACE::IoExc &)
 		{
 		    tileBroken = true;		// tile cannot be decoded
 		}
@@ -571,7 +575,7 @@ writeReadIncomplete ()
 
         cout << "reading multiple tiles at a time," << flush;
 
-        TiledRgbaInputFile in (fileName);
+        TiledRgbaInputFile in (fileName.c_str());
         const Box2i &dw = in.dataWindow();
 
         assert (dw.max.x - dw.min.x + 1 == width);
@@ -590,11 +594,11 @@ writeReadIncomplete ()
 	    {
 		in.readTiles (0, in.numXTiles() - 1, tileY, tileY);
 	    }
-	    catch (const Iex::InputExc &)
+	    catch (const IEX_NAMESPACE::InputExc &)
 	    {
 		tilesMissing = true;
 	    }
-	    catch (const Iex::IoExc &)
+	    catch (const IEX_NAMESPACE::IoExc &)
 	    {
 		tilesBroken = true;
 	    }
@@ -617,24 +621,248 @@ writeReadIncomplete ()
 	}
     }
 
-    remove (fileName);
+    remove (fileName.c_str());
+}
+
+
+void
+writeReadLayers(const std::string &tempDir)
+{
+    cout << "\nreading multi-layer file" << endl;
+
+    std::string fileName = tempDir + "imf_test_tiled_multi_layer_rgba.exr";
+
+    const int W = 237;
+    const int H = 119;
+    
+    Array2D<half> p1 (H, W);
+    Array2D<half> p2 (H, W);
+
+    for (int y = 0; y < H; ++y)
+    {
+	for (int x = 0; x < W; ++x)
+	{
+	    p1[y][x] = half (y % 23 + x % 17);
+	    p2[y][x] = half (y % 29 + x % 19);
+	}
+    }
+
+    {
+	Header hdr (W, H);
+	hdr.setTileDescription (TileDescription());
+	hdr.channels().insert ("R", Channel (HALF));
+	hdr.channels().insert ("foo.R", Channel (HALF));
+
+	FrameBuffer fb;
+
+	fb.insert ("R",
+		   Slice (HALF,			// type
+			  (char *) &p1[0][0], 	// base
+			  sizeof (half),	// xStride
+			  sizeof (half) * W));	// yStride
+
+	fb.insert ("foo.R",
+		   Slice (HALF,			// type
+			  (char *) &p2[0][0], 	// base
+			  sizeof (half),	// xStride
+			  sizeof (half) * W));	// yStride
+
+	TiledOutputFile out (fileName.c_str(), hdr);
+	out.setFrameBuffer (fb);
+	out.writeTiles (0, out.numXTiles() - 1, 0, out.numYTiles() - 1);
+    }
+
+    {
+	TiledRgbaInputFile in (fileName.c_str(), "");
+
+	Array2D<Rgba> p3 (H, W);
+	in.setFrameBuffer (&p3[0][0], 1, W);
+	in.readTiles (0, in.numXTiles() - 1, 0, in.numYTiles() - 1);
+
+	for (int y = 0; y < H; ++y)
+	{
+	    for (int x = 0; x < W; ++x)
+	    {
+		assert (p3[y][x].r == p1[y][x]);
+		assert (p3[y][x].g == 0);
+		assert (p3[y][x].b == 0);
+		assert (p3[y][x].a == 1);
+	    }
+	}
+    }
+
+    {
+	TiledRgbaInputFile in (fileName.c_str(), "foo");
+
+	Array2D<Rgba> p3 (H, W);
+	in.setFrameBuffer (&p3[0][0], 1, W);
+	in.readTiles (0, in.numXTiles() - 1, 0, in.numYTiles() - 1);
+
+	for (int y = 0; y < H; ++y)
+	{
+	    for (int x = 0; x < W; ++x)
+	    {
+		assert (p3[y][x].r == p2[y][x]);
+		assert (p3[y][x].g == 0);
+		assert (p3[y][x].b == 0);
+		assert (p3[y][x].a == 1);
+	    }
+	}
+    }
+
+    {
+	TiledRgbaInputFile in (fileName.c_str(), "");
+
+	Array2D<Rgba> p3 (H, W);
+
+	in.setFrameBuffer (&p3[0][0], 1, W);
+
+	in.readTiles (0, in.numXTiles() - 1,
+		      0, in.numYTiles() / 2 - 1);
+
+	in.setLayerName ("foo");
+
+	in.setFrameBuffer (&p3[0][0], 1, W);
+
+	in.readTiles (0, in.numXTiles() - 1,
+		      in.numYTiles() / 2, in.numYTiles() - 1);
+
+	for (int y = 0; y < H; ++y)
+	{
+	    for (int x = 0; x < W; ++x)
+	    {
+		if (y < in.numYTiles() / 2 * in.tileYSize())
+		    assert (p3[y][x].r == p1[y][x]);
+		else
+		    assert (p3[y][x].r == p2[y][x]);
+
+		assert (p3[y][x].g == 0);
+		assert (p3[y][x].b == 0);
+		assert (p3[y][x].a == 1);
+	    }
+	}
+    }
+
+    {
+	Header hdr (W, H);
+	hdr.setTileDescription (TileDescription());
+	hdr.channels().insert ("Y", Channel (HALF));
+	hdr.channels().insert ("foo.Y", Channel (HALF));
+
+	FrameBuffer fb;
+
+	fb.insert ("Y",
+		   Slice (HALF,			// type
+			  (char *) &p1[0][0], 	// base
+			  sizeof (half),	// xStride
+			  sizeof (half) * W));	// yStride
+
+	fb.insert ("foo.Y",
+		   Slice (HALF,			// type
+			  (char *) &p2[0][0], 	// base
+			  sizeof (half),	// xStride
+			  sizeof (half) * W));	// yStride
+
+	TiledOutputFile out (fileName.c_str(), hdr);
+	out.setFrameBuffer (fb);
+	out.writeTiles (0, out.numXTiles() - 1, 0, out.numYTiles() - 1);
+    }
+
+    {
+	TiledRgbaInputFile in (fileName.c_str(), "");
+
+	Array2D<Rgba> p3 (H, W);
+	in.setFrameBuffer (&p3[0][0], 1, W);
+	in.readTiles (0, in.numXTiles() - 1, 0, in.numYTiles() - 1);
+
+	for (int y = 0; y < H; ++y)
+	{
+	    for (int x = 0; x < W; ++x)
+	    {
+		assert (p3[y][x].r == p1[y][x]);
+		assert (p3[y][x].g == p1[y][x]);
+		assert (p3[y][x].b == p1[y][x]);
+		assert (p3[y][x].a == 1);
+	    }
+	}
+    }
+
+    {
+	TiledRgbaInputFile in (fileName.c_str(), "foo");
+
+	Array2D<Rgba> p3 (H, W);
+	in.setFrameBuffer (&p3[0][0], 1, W);
+	in.readTiles (0, in.numXTiles() - 1, 0, in.numYTiles() - 1);
+
+	for (int y = 0; y < H; ++y)
+	{
+	    for (int x = 0; x < W; ++x)
+	    {
+		assert (p3[y][x].r == p2[y][x]);
+		assert (p3[y][x].g == p2[y][x]);
+		assert (p3[y][x].b == p2[y][x]);
+		assert (p3[y][x].a == 1);
+	    }
+	}
+    }
+
+    {
+	TiledRgbaInputFile in (fileName.c_str(), "");
+
+	Array2D<Rgba> p3 (H, W);
+
+	in.setFrameBuffer (&p3[0][0], 1, W);
+
+	in.readTiles (0, in.numXTiles() - 1,
+		      0, in.numYTiles() / 2 - 1);
+
+	in.setLayerName ("foo");
+
+	in.setFrameBuffer (&p3[0][0], 1, W);
+
+	in.readTiles (0, in.numXTiles() - 1,
+		      in.numYTiles() / 2, in.numYTiles() - 1);
+
+	for (int y = 0; y < H; ++y)
+	{
+	    for (int x = 0; x < W; ++x)
+	    {
+		if (y < in.numYTiles() / 2 * in.tileYSize())
+		{
+		    assert (p3[y][x].r == p1[y][x]);
+		    assert (p3[y][x].g == p1[y][x]);
+		    assert (p3[y][x].b == p1[y][x]);
+		}
+		else
+		{
+		    assert (p3[y][x].r == p2[y][x]);
+		    assert (p3[y][x].g == p2[y][x]);
+		    assert (p3[y][x].b == p2[y][x]);
+		}
+
+		assert (p3[y][x].a == 1);
+	    }
+	}
+    }
+
+    remove (fileName.c_str());
 }
 
 } // namespace
 
 
 void
-testTiledRgba ()
+testTiledRgba (const std::string &tempDir)
 {
     try
     {
         cout << "Testing the tiled RGBA image interface" << endl;
 
-	int maxThreads = IlmThread::supportsThreads()? 3: 0;
+	int maxThreads = ILMTHREAD_NAMESPACE::supportsThreads()? 3: 0;
 
 	for (int n = 0; n <= maxThreads; ++n)
 	{
-	    if (IlmThread::supportsThreads())
+	    if (ILMTHREAD_NAMESPACE::supportsThreads())
 	    {
 		setGlobalThreadCount (n);
 		cout << "\nnumber of threads: " << globalThreadCount() << endl;
@@ -664,18 +892,20 @@ testTiledRgba ()
 			// tiles are rather slow anyway)
 			//
 
-			writeRead (W[i], H[i], Compression (comp), 1, 1);
+			writeRead (tempDir, W[i], H[i], Compression (comp), 1, 1);
 		    }
 
-		    writeRead (W[i], H[i], Compression (comp), 35, 26);
-		    writeRead (W[i], H[i], Compression (comp), 75, 52);
-		    writeRead (W[i], H[i], Compression (comp), 264, 129);
+		    writeRead (tempDir, W[i], H[i], Compression (comp), 35, 26);
+		    writeRead (tempDir, W[i], H[i], Compression (comp), 75, 52);
+		    writeRead (tempDir, W[i], H[i], Compression (comp), 264, 129);
 		}
 	    }
 
-	    writeReadIncomplete();
+	    writeReadIncomplete (tempDir);
 	}
 
+	writeReadLayers (tempDir);
+
         cout << "ok\n" << endl;
     }
     catch (const std::exception &e)
